# 
# Generated by the nebius.base.protos.compiler.  DO NOT EDIT!
# 

"""Auto-generated Nebius SDK package for ``nebius.mysterybox.v1``"""

import builtins as builtins
import collections.abc as abc
import datetime as datetime
import google.protobuf.descriptor as descriptor_1
import google.protobuf.message as message_1
import google.protobuf.timestamp_pb2 as timestamp_pb2
import nebius.aio.client as client
import nebius.aio.operation as operation
import nebius.aio.request as request_1
import nebius.aio.request_kwargs as request_kwargs
import nebius.api.nebius.common.v1 as v1_1
import nebius.api.nebius.common.v1.metadata_pb2 as metadata_pb2
import nebius.api.nebius.common.v1.operation_pb2 as operation_pb2
import nebius.api.nebius.mysterybox.v1.payload_pb2 as payload_pb2
import nebius.api.nebius.mysterybox.v1.payload_service_pb2 as payload_service_pb2
import nebius.api.nebius.mysterybox.v1.secret_pb2 as secret_pb2
import nebius.api.nebius.mysterybox.v1.secret_service_pb2 as secret_service_pb2
import nebius.api.nebius.mysterybox.v1.secret_version_pb2 as secret_version_pb2
import nebius.api.nebius.mysterybox.v1.secret_version_service_pb2 as secret_version_service_pb2
import nebius.base.fieldmask_protobuf as fieldmask_protobuf
import nebius.base.protos.descriptor as descriptor
import nebius.base.protos.pb_classes as pb_classes
import nebius.base.protos.pb_enum as pb_enum
import nebius.base.protos.unset as unset
import nebius.base.protos.well_known as well_known_1
import typing as typing
import typing_extensions as typing_extensions
#@ local imports here @#

# file: nebius/mysterybox/v1/payload.proto
class Payload(pb_classes.Message):
    """
    Represents the secret's content as key-value pair.
    """
    
    __PB2_CLASS__ = payload_pb2.Payload
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.Payload",payload_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    class __OneOfClass_payload__(pb_classes.OneOf):
        name: builtins.str= "payload"
        
        def __init__(self, msg: "Payload") -> None:
            super().__init__()
            self._message: "Payload" = msg
    
    class __OneOfClass_payload_string_value__(__OneOfClass_payload__):
        field: typing.Literal["string_value"] = "string_value"
        
        def __init__(self, msg: "Payload") -> None:
            super().__init__(msg)
        @builtins.property
        def value(self) -> "builtins.str":
            return self._message.string_value
    
    class __OneOfClass_payload_binary_value__(__OneOfClass_payload__):
        field: typing.Literal["binary_value"] = "binary_value"
        
        def __init__(self, msg: "Payload") -> None:
            super().__init__(msg)
        @builtins.property
        def value(self) -> "builtins.bytes":
            return self._message.binary_value
    
    @builtins.property
    def payload(self) -> __OneOfClass_payload_string_value__|__OneOfClass_payload_binary_value__|None:
        field_name_1: str|None = super().which_field_in_oneof("payload")
        match field_name_1:
            case "string_value":
                return self.__OneOfClass_payload_string_value__(self)
            case "binary_value":
                return self.__OneOfClass_payload_binary_value__(self)
            case None:
                return None
            case _:
                raise pb_classes.OneOfMatchError(field_name_1)
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        key: "builtins.str|None|unset.UnsetType" = unset.Unset,
        string_value: "builtins.str|None|unset.UnsetType" = unset.Unset,
        binary_value: "builtins.bytes|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(key, unset.UnsetType):
            self.key = key
        if not isinstance(string_value, unset.UnsetType):
            self.string_value = string_value
        if not isinstance(binary_value, unset.UnsetType):
            self.binary_value = binary_value
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "key",
            "string_value",
            "binary_value",
            "payload",
        ]
    
    @builtins.property
    def key(self) -> "builtins.str":
        """
        Non-confidential key of the payload entry.
        """
        
        return super()._get_field("key", explicit_presence=False,
        )
    @key.setter
    def key(self, value: "builtins.str|None") -> None:
        return super()._set_field("key",value,explicit_presence=False,
        )
    
    @builtins.property
    def string_value(self) -> "builtins.str|None":
        """
        The text data to encrypt and store in the version of the secret.
        """
        
        return super()._get_field("string_value", explicit_presence=True,
        )
    @string_value.setter
    def string_value(self, value: "builtins.str|None") -> None:
        return super()._set_field("string_value",value,explicit_presence=True,
        )
    
    @builtins.property
    def binary_value(self) -> "builtins.bytes|None":
        """
        The binary data to encrypt and store in the version of the secret.
        """
        
        return super()._get_field("binary_value", explicit_presence=True,
        )
    @binary_value.setter
    def binary_value(self, value: "builtins.bytes|None") -> None:
        return super()._set_field("binary_value",value,explicit_presence=True,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "key":"key",
        "string_value":"string_value",
        "binary_value":"binary_value",
        "payload":"payload",
    }
    
# file: nebius/mysterybox/v1/payload_service.proto
class GetPayloadRequest(pb_classes.Message):
    __PB2_CLASS__ = payload_service_pb2.GetPayloadRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.GetPayloadRequest",payload_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        secret_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        version_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(secret_id, unset.UnsetType):
            self.secret_id = secret_id
        if not isinstance(version_id, unset.UnsetType):
            self.version_id = version_id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "secret_id",
            "version_id",
        ]
    
    @builtins.property
    def secret_id(self) -> "builtins.str":
        return super()._get_field("secret_id", explicit_presence=False,
        )
    @secret_id.setter
    def secret_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("secret_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def version_id(self) -> "builtins.str":
        """
        Desired version of the secret. If not provided, the primary version of the secret will be returned.
        """
        
        return super()._get_field("version_id", explicit_presence=False,
        )
    @version_id.setter
    def version_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("version_id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "secret_id":"secret_id",
        "version_id":"version_id",
    }
    
class GetPayloadByKeyRequest(pb_classes.Message):
    __PB2_CLASS__ = payload_service_pb2.GetPayloadByKeyRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.GetPayloadByKeyRequest",payload_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        secret_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        version_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        key: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(secret_id, unset.UnsetType):
            self.secret_id = secret_id
        if not isinstance(version_id, unset.UnsetType):
            self.version_id = version_id
        if not isinstance(key, unset.UnsetType):
            self.key = key
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "secret_id",
            "version_id",
            "key",
        ]
    
    @builtins.property
    def secret_id(self) -> "builtins.str":
        return super()._get_field("secret_id", explicit_presence=False,
        )
    @secret_id.setter
    def secret_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("secret_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def version_id(self) -> "builtins.str":
        """
        Desired version of the secret. If not provided, the primary version of the secret will be returned.
        """
        
        return super()._get_field("version_id", explicit_presence=False,
        )
    @version_id.setter
    def version_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("version_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def key(self) -> "builtins.str":
        """
        Payload's key to be retrieved.
        """
        
        return super()._get_field("key", explicit_presence=False,
        )
    @key.setter
    def key(self, value: "builtins.str|None") -> None:
        return super()._set_field("key",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "secret_id":"secret_id",
        "version_id":"version_id",
        "key":"key",
    }
    
class SecretPayload(pb_classes.Message):
    __PB2_CLASS__ = payload_service_pb2.SecretPayload
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.SecretPayload",payload_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        version_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        data: "abc.Iterable[Payload]|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(version_id, unset.UnsetType):
            self.version_id = version_id
        if not isinstance(data, unset.UnsetType):
            self.data = data
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "version_id",
            "data",
        ]
    
    @builtins.property
    def version_id(self) -> "builtins.str":
        """
        Secret's version related to returned payload.
        """
        
        return super()._get_field("version_id", explicit_presence=False,
        )
    @version_id.setter
    def version_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("version_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def data(self) -> "abc.MutableSequence[Payload]":
        return super()._get_field("data", explicit_presence=False,
        wrap=pb_classes.Repeated.with_wrap(Payload,None,None),
        )
    @data.setter
    def data(self, value: "abc.Iterable[Payload]|None") -> None:
        return super()._set_field("data",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "version_id":"version_id",
        "data":"data",
    }
    
class SecretPayloadEntry(pb_classes.Message):
    __PB2_CLASS__ = payload_service_pb2.SecretPayloadEntry
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.SecretPayloadEntry",payload_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        version_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        data: "Payload|payload_pb2.Payload|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(version_id, unset.UnsetType):
            self.version_id = version_id
        if not isinstance(data, unset.UnsetType):
            self.data = data
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "version_id",
            "data",
        ]
    
    @builtins.property
    def version_id(self) -> "builtins.str":
        """
        Secret's version related to returned payload.
        """
        
        return super()._get_field("version_id", explicit_presence=False,
        )
    @version_id.setter
    def version_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("version_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def data(self) -> "Payload":
        return super()._get_field("data", explicit_presence=False,
        wrap=Payload,
        )
    @data.setter
    def data(self, value: "Payload|payload_pb2.Payload|None") -> None:
        return super()._set_field("data",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "version_id":"version_id",
        "data":"data",
    }
    

class PayloadServiceClient(client.Client):
    """
    This class provides the client methods for the ``.nebius.mysterybox.v1.PayloadService`` service.
    
    Each method constructs a :class:`nebius.aio.request.Request` object
    that represents the in-flight RPC. The request can be awaited (async)
    or waited synchronously using its ``.wait()`` helpers.
    
    The request methods accept various parameters to configure metadata,
    timeouts, authorization, and retries. See individual method docstrings
    for details.
    
    :cvar __service_name__: The full protobuf service name.
    """
    
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.ServiceDescriptor](".nebius.mysterybox.v1.PayloadService",payload_service_pb2.DESCRIPTOR,descriptor_1.ServiceDescriptor)
    """The protobuf service descriptor extraction function."""
    __service_name__ = ".nebius.mysterybox.v1.PayloadService"
    
    def get(self,
        request: "GetPayloadRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["GetPayloadRequest","SecretPayload"]:
        """
        Returns the secret's payload in its decrypted format.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.mysterybox.v1.GetPayloadRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.mysterybox.v1.SecretPayload`.
        """
        
        return super().request(
            method="Get",
            request=request,
            result_pb2_class=payload_service_pb2.SecretPayload,
            result_wrapper=pb_classes.simple_wrapper(SecretPayload),
            **kwargs,
        )
    
    def get_by_key(self,
        request: "GetPayloadByKeyRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["GetPayloadByKeyRequest","SecretPayloadEntry"]:
        """
        Returns the secret's payload value by key in its decrypted format.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.mysterybox.v1.GetPayloadByKeyRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.mysterybox.v1.SecretPayloadEntry`.
        """
        
        return super().request(
            method="GetByKey",
            request=request,
            result_pb2_class=payload_service_pb2.SecretPayloadEntry,
            result_wrapper=pb_classes.simple_wrapper(SecretPayloadEntry),
            **kwargs,
        )
    

# file: nebius/mysterybox/v1/secret_version.proto
class SecretVersion(pb_classes.Message):
    __PB2_CLASS__ = secret_version_pb2.SecretVersion
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.SecretVersion",secret_version_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        metadata: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None|unset.UnsetType" = unset.Unset,
        spec: "SecretVersionSpec|secret_version_pb2.SecretVersionSpec|None|unset.UnsetType" = unset.Unset,
        status: "SecretVersionStatus|secret_version_pb2.SecretVersionStatus|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(metadata, unset.UnsetType):
            self.metadata = metadata
        if not isinstance(spec, unset.UnsetType):
            self.spec = spec
        if not isinstance(status, unset.UnsetType):
            self.status = status
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "metadata",
            "spec",
            "status",
        ]
    
    @builtins.property
    def metadata(self) -> "v1_1.ResourceMetadata":
        return super()._get_field("metadata", explicit_presence=False,
        wrap=v1_1.ResourceMetadata,
        )
    @metadata.setter
    def metadata(self, value: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None") -> None:
        return super()._set_field("metadata",value,explicit_presence=False,
        )
    
    @builtins.property
    def spec(self) -> "SecretVersionSpec":
        return super()._get_field("spec", explicit_presence=False,
        wrap=SecretVersionSpec,
        )
    @spec.setter
    def spec(self, value: "SecretVersionSpec|secret_version_pb2.SecretVersionSpec|None") -> None:
        return super()._set_field("spec",value,explicit_presence=False,
        )
    
    @builtins.property
    def status(self) -> "SecretVersionStatus":
        """
        The status of the secret version.
        """
        
        return super()._get_field("status", explicit_presence=False,
        wrap=SecretVersionStatus,
        )
    @status.setter
    def status(self, value: "SecretVersionStatus|secret_version_pb2.SecretVersionStatus|None") -> None:
        return super()._set_field("status",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "metadata":"metadata",
        "spec":"spec",
        "status":"status",
    }
    
class SecretVersionSpec(pb_classes.Message):
    __PB2_CLASS__ = secret_version_pb2.SecretVersionSpec
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.SecretVersionSpec",secret_version_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        description: "builtins.str|None|unset.UnsetType" = unset.Unset,
        payload: "abc.Iterable[Payload]|None|unset.UnsetType" = unset.Unset,
        set_primary: "builtins.bool|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(description, unset.UnsetType):
            self.description = description
        if not isinstance(payload, unset.UnsetType):
            self.payload = payload
        if not isinstance(set_primary, unset.UnsetType):
            self.set_primary = set_primary
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "description",
            "payload",
            "set_primary",
        ]
    
    @builtins.property
    def description(self) -> "builtins.str":
        """
        Description of the version.
        """
        
        return super()._get_field("description", explicit_presence=False,
        )
    @description.setter
    def description(self, value: "builtins.str|None") -> None:
        return super()._set_field("description",value,explicit_presence=False,
        )
    
    @builtins.property
    def payload(self) -> "abc.MutableSequence[Payload]":
        """
        Payload entries related to the version.
        """
        
        return super()._get_field("payload", explicit_presence=False,
        wrap=pb_classes.Repeated.with_wrap(Payload,None,None),
        )
    @payload.setter
    def payload(self, value: "abc.Iterable[Payload]|None") -> None:
        return super()._set_field("payload",value,explicit_presence=False,
        )
    
    @builtins.property
    def set_primary(self) -> "builtins.bool":
        return super()._get_field("set_primary", explicit_presence=False,
        )
    @set_primary.setter
    def set_primary(self, value: "builtins.bool|None") -> None:
        return super()._set_field("set_primary",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "description":"description",
        "payload":"payload",
        "set_primary":"set_primary",
    }
    
class SecretVersionStatus(pb_classes.Message):
    __PB2_CLASS__ = secret_version_pb2.SecretVersionStatus
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.SecretVersionStatus",secret_version_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
        "deleted_at": well_known_1.ts_mask,
        "purge_at": well_known_1.ts_mask,
    }
    
    class State(pb_enum.Enum):
        __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.EnumDescriptor](".nebius.mysterybox.v1.SecretVersionStatus.State",secret_version_pb2.DESCRIPTOR,descriptor_1.EnumDescriptor)
        STATE_UNSPECIFIED = 0
        ACTIVE = 1
        """
        Resource is active, ready for use
        """
        
        SCHEDULED_FOR_DELETION = 2
        """
        Resource was marked as soft deleted
        """
        
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        state: "SecretVersionStatus.State|secret_version_pb2.SecretVersionStatus.State|None|unset.UnsetType" = unset.Unset,
        deleted_at: "timestamp_pb2.Timestamp|datetime.datetime|None|unset.UnsetType" = unset.Unset,
        purge_at: "timestamp_pb2.Timestamp|datetime.datetime|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(state, unset.UnsetType):
            self.state = state
        if not isinstance(deleted_at, unset.UnsetType):
            self.deleted_at = deleted_at
        if not isinstance(purge_at, unset.UnsetType):
            self.purge_at = purge_at
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "state",
            "deleted_at",
            "purge_at",
            "State",
        ]
    
    @builtins.property
    def state(self) -> "SecretVersionStatus.State":
        """
        State (ACTIVE, SCHEDULED_FOR_DELETION)
        """
        
        return super()._get_field("state", explicit_presence=False,
        wrap=SecretVersionStatus.State,
        )
    @state.setter
    def state(self, value: "SecretVersionStatus.State|secret_version_pb2.SecretVersionStatus.State|None") -> None:
        return super()._set_field("state",value,explicit_presence=False,
        )
    
    @builtins.property
    def deleted_at(self) -> "datetime.datetime":
        """
        Time when user called soft delete method
        """
        
        return super()._get_field("deleted_at", explicit_presence=False,
        wrap=well_known_1.from_timestamp
        )
    @deleted_at.setter
    def deleted_at(self, value: "timestamp_pb2.Timestamp|datetime.datetime|None") -> None:
        return super()._set_field("deleted_at",value,explicit_presence=False,
        unwrap=well_known_1.to_timestamp
        )
    
    @builtins.property
    def purge_at(self) -> "datetime.datetime":
        """
        Time when key should be totally deleted from DB
        """
        
        return super()._get_field("purge_at", explicit_presence=False,
        wrap=well_known_1.from_timestamp
        )
    @purge_at.setter
    def purge_at(self, value: "timestamp_pb2.Timestamp|datetime.datetime|None") -> None:
        return super()._set_field("purge_at",value,explicit_presence=False,
        unwrap=well_known_1.to_timestamp
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "state":"state",
        "deleted_at":"deleted_at",
        "purge_at":"purge_at",
        "State":"State",
    }
    
# file: nebius/mysterybox/v1/secret.proto
class Secret(pb_classes.Message):
    __PB2_CLASS__ = secret_pb2.Secret
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.Secret",secret_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        metadata: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None|unset.UnsetType" = unset.Unset,
        spec: "SecretSpec|secret_pb2.SecretSpec|None|unset.UnsetType" = unset.Unset,
        status: "SecretStatus|secret_pb2.SecretStatus|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(metadata, unset.UnsetType):
            self.metadata = metadata
        if not isinstance(spec, unset.UnsetType):
            self.spec = spec
        if not isinstance(status, unset.UnsetType):
            self.status = status
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "metadata",
            "spec",
            "status",
        ]
    
    @builtins.property
    def metadata(self) -> "v1_1.ResourceMetadata":
        return super()._get_field("metadata", explicit_presence=False,
        wrap=v1_1.ResourceMetadata,
        )
    @metadata.setter
    def metadata(self, value: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None") -> None:
        return super()._set_field("metadata",value,explicit_presence=False,
        )
    
    @builtins.property
    def spec(self) -> "SecretSpec":
        """
        The specifications for creating a secret.
        """
        
        return super()._get_field("spec", explicit_presence=False,
        wrap=SecretSpec,
        )
    @spec.setter
    def spec(self, value: "SecretSpec|secret_pb2.SecretSpec|None") -> None:
        return super()._set_field("spec",value,explicit_presence=False,
        )
    
    @builtins.property
    def status(self) -> "SecretStatus":
        """
        The status of the secret.
        """
        
        return super()._get_field("status", explicit_presence=False,
        wrap=SecretStatus,
        )
    @status.setter
    def status(self, value: "SecretStatus|secret_pb2.SecretStatus|None") -> None:
        return super()._set_field("status",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "metadata":"metadata",
        "spec":"spec",
        "status":"status",
    }
    
class SecretSpec(pb_classes.Message):
    __PB2_CLASS__ = secret_pb2.SecretSpec
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.SecretSpec",secret_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    class __OneOfClass__primary_version_id__(pb_classes.OneOf):
        name: builtins.str= "_primary_version_id"
        
        def __init__(self, msg: "SecretSpec") -> None:
            super().__init__()
            self._message: "SecretSpec" = msg
    
    class __OneOfClass__primary_version_id_primary_version_id__(__OneOfClass__primary_version_id__):
        field: typing.Literal["primary_version_id"] = "primary_version_id"
        
        def __init__(self, msg: "SecretSpec") -> None:
            super().__init__(msg)
        @builtins.property
        def value(self) -> "builtins.str":
            return self._message.primary_version_id
    
    @builtins.property
    def _primary_version_id(self) -> __OneOfClass__primary_version_id_primary_version_id__|None:
        field_name_1: str|None = super().which_field_in_oneof("_primary_version_id")
        match field_name_1:
            case "primary_version_id":
                return self.__OneOfClass__primary_version_id_primary_version_id__(self)
            case None:
                return None
            case _:
                raise pb_classes.OneOfMatchError(field_name_1)
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        description: "builtins.str|None|unset.UnsetType" = unset.Unset,
        primary_version_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        secret_version: "SecretVersionSpec|secret_version_pb2.SecretVersionSpec|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(description, unset.UnsetType):
            self.description = description
        if not isinstance(primary_version_id, unset.UnsetType):
            self.primary_version_id = primary_version_id
        if not isinstance(secret_version, unset.UnsetType):
            self.secret_version = secret_version
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "description",
            "primary_version_id",
            "secret_version",
            "_primary_version_id",
        ]
    
    @builtins.property
    def description(self) -> "builtins.str":
        """
        Description of the secret.
        """
        
        return super()._get_field("description", explicit_presence=False,
        )
    @description.setter
    def description(self, value: "builtins.str|None") -> None:
        return super()._set_field("description",value,explicit_presence=False,
        )
    
    @builtins.property
    def primary_version_id(self) -> "builtins.str|None":
        """
        Specifies the primary version of the secret to update its payload. This parameter should only be provided during update operations.
        """
        
        return super()._get_field("primary_version_id", explicit_presence=True,
        )
    @primary_version_id.setter
    def primary_version_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("primary_version_id",value,explicit_presence=True,
        )
    
    @builtins.property
    def secret_version(self) -> "SecretVersionSpec":
        """
        Secret's version specification, defines the secret version, including its payload. This parameter must be specified only during create operations.
        """
        
        return super()._get_field("secret_version", explicit_presence=False,
        wrap=SecretVersionSpec,
        )
    @secret_version.setter
    def secret_version(self, value: "SecretVersionSpec|secret_version_pb2.SecretVersionSpec|None") -> None:
        return super()._set_field("secret_version",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "description":"description",
        "primary_version_id":"primary_version_id",
        "secret_version":"secret_version",
        "_primary_version_id":"_primary_version_id",
    }
    
class SecretStatus(pb_classes.Message):
    __PB2_CLASS__ = secret_pb2.SecretStatus
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.SecretStatus",secret_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
        "deleted_at": well_known_1.ts_mask,
        "purge_at": well_known_1.ts_mask,
    }
    
    class State(pb_enum.Enum):
        __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.EnumDescriptor](".nebius.mysterybox.v1.SecretStatus.State",secret_pb2.DESCRIPTOR,descriptor_1.EnumDescriptor)
        STATE_UNSPECIFIED = 0
        ACTIVE = 1
        """
        Resource is active, ready for use
        """
        
        SCHEDULED_FOR_DELETION = 2
        """
        Resource was marked as soft deleted
        """
        
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        state: "SecretStatus.State|secret_pb2.SecretStatus.State|None|unset.UnsetType" = unset.Unset,
        deleted_at: "timestamp_pb2.Timestamp|datetime.datetime|None|unset.UnsetType" = unset.Unset,
        purge_at: "timestamp_pb2.Timestamp|datetime.datetime|None|unset.UnsetType" = unset.Unset,
        effective_kms_key_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(state, unset.UnsetType):
            self.state = state
        if not isinstance(deleted_at, unset.UnsetType):
            self.deleted_at = deleted_at
        if not isinstance(purge_at, unset.UnsetType):
            self.purge_at = purge_at
        if not isinstance(effective_kms_key_id, unset.UnsetType):
            self.effective_kms_key_id = effective_kms_key_id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "state",
            "deleted_at",
            "purge_at",
            "effective_kms_key_id",
            "State",
        ]
    
    @builtins.property
    def state(self) -> "SecretStatus.State":
        """
        State (ACTIVE, SCHEDULED_FOR_DELETION)
        """
        
        return super()._get_field("state", explicit_presence=False,
        wrap=SecretStatus.State,
        )
    @state.setter
    def state(self, value: "SecretStatus.State|secret_pb2.SecretStatus.State|None") -> None:
        return super()._set_field("state",value,explicit_presence=False,
        )
    
    @builtins.property
    def deleted_at(self) -> "datetime.datetime":
        """
        Time when user called soft delete method
        """
        
        return super()._get_field("deleted_at", explicit_presence=False,
        wrap=well_known_1.from_timestamp
        )
    @deleted_at.setter
    def deleted_at(self, value: "timestamp_pb2.Timestamp|datetime.datetime|None") -> None:
        return super()._set_field("deleted_at",value,explicit_presence=False,
        unwrap=well_known_1.to_timestamp
        )
    
    @builtins.property
    def purge_at(self) -> "datetime.datetime":
        """
        Time when key should be totally deleted from DB
        """
        
        return super()._get_field("purge_at", explicit_presence=False,
        wrap=well_known_1.from_timestamp
        )
    @purge_at.setter
    def purge_at(self, value: "timestamp_pb2.Timestamp|datetime.datetime|None") -> None:
        return super()._set_field("purge_at",value,explicit_presence=False,
        unwrap=well_known_1.to_timestamp
        )
    
    @builtins.property
    def effective_kms_key_id(self) -> "builtins.str":
        return super()._get_field("effective_kms_key_id", explicit_presence=False,
        )
    @effective_kms_key_id.setter
    def effective_kms_key_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("effective_kms_key_id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "state":"state",
        "deleted_at":"deleted_at",
        "purge_at":"purge_at",
        "effective_kms_key_id":"effective_kms_key_id",
        "State":"State",
    }
    
# file: nebius/mysterybox/v1/secret_service.proto
class CreateSecretRequest(pb_classes.Message):
    __PB2_CLASS__ = secret_service_pb2.CreateSecretRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.CreateSecretRequest",secret_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        metadata: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None|unset.UnsetType" = unset.Unset,
        spec: "SecretSpec|secret_pb2.SecretSpec|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(metadata, unset.UnsetType):
            self.metadata = metadata
        if not isinstance(spec, unset.UnsetType):
            self.spec = spec
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "metadata",
            "spec",
        ]
    
    @builtins.property
    def metadata(self) -> "v1_1.ResourceMetadata":
        return super()._get_field("metadata", explicit_presence=False,
        wrap=v1_1.ResourceMetadata,
        )
    @metadata.setter
    def metadata(self, value: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None") -> None:
        return super()._set_field("metadata",value,explicit_presence=False,
        )
    
    @builtins.property
    def spec(self) -> "SecretSpec":
        return super()._get_field("spec", explicit_presence=False,
        wrap=SecretSpec,
        )
    @spec.setter
    def spec(self, value: "SecretSpec|secret_pb2.SecretSpec|None") -> None:
        return super()._set_field("spec",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "metadata":"metadata",
        "spec":"spec",
    }
    
class UpdateSecretRequest(pb_classes.Message):
    __PB2_CLASS__ = secret_service_pb2.UpdateSecretRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.UpdateSecretRequest",secret_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        metadata: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None|unset.UnsetType" = unset.Unset,
        spec: "SecretSpec|secret_pb2.SecretSpec|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(metadata, unset.UnsetType):
            self.metadata = metadata
        if not isinstance(spec, unset.UnsetType):
            self.spec = spec
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "metadata",
            "spec",
        ]
    
    @builtins.property
    def metadata(self) -> "v1_1.ResourceMetadata":
        return super()._get_field("metadata", explicit_presence=False,
        wrap=v1_1.ResourceMetadata,
        )
    @metadata.setter
    def metadata(self, value: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None") -> None:
        return super()._set_field("metadata",value,explicit_presence=False,
        )
    
    @builtins.property
    def spec(self) -> "SecretSpec":
        return super()._get_field("spec", explicit_presence=False,
        wrap=SecretSpec,
        )
    @spec.setter
    def spec(self, value: "SecretSpec|secret_pb2.SecretSpec|None") -> None:
        return super()._set_field("spec",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "metadata":"metadata",
        "spec":"spec",
    }
    
class GetSecretRequest(pb_classes.Message):
    __PB2_CLASS__ = secret_service_pb2.GetSecretRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.GetSecretRequest",secret_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        show_scheduled_for_deletion: "builtins.bool|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
        if not isinstance(show_scheduled_for_deletion, unset.UnsetType):
            self.show_scheduled_for_deletion = show_scheduled_for_deletion
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
            "show_scheduled_for_deletion",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        """
        ID of secret to return.
        """
        
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    @builtins.property
    def show_scheduled_for_deletion(self) -> "builtins.bool":
        """
        By default, Get doesn't return resource if it is scheduled for deletion.
        If show_scheduled_for_deletion = true, the Get operation returns the resource even if it is scheduled for deletion.
        If show_scheduled_for_deletion = false, the Get method returns the NOT_FOUND gRPC status code.
        """
        
        return super()._get_field("show_scheduled_for_deletion", explicit_presence=False,
        )
    @show_scheduled_for_deletion.setter
    def show_scheduled_for_deletion(self, value: "builtins.bool|None") -> None:
        return super()._set_field("show_scheduled_for_deletion",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
        "show_scheduled_for_deletion":"show_scheduled_for_deletion",
    }
    
class GetSecretByNameRequest(pb_classes.Message):
    __PB2_CLASS__ = secret_service_pb2.GetSecretByNameRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.GetSecretByNameRequest",secret_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        parent_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(parent_id, unset.UnsetType):
            self.parent_id = parent_id
        if not isinstance(name, unset.UnsetType):
            self.name = name
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "parent_id",
            "name",
        ]
    
    @builtins.property
    def parent_id(self) -> "builtins.str":
        """
        ParentId and name of the secret to return.
        """
        
        return super()._get_field("parent_id", explicit_presence=False,
        )
    @parent_id.setter
    def parent_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("parent_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def name(self) -> "builtins.str":
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "parent_id":"parent_id",
        "name":"name",
    }
    
class ListSecretsRequest(pb_classes.Message):
    __PB2_CLASS__ = secret_service_pb2.ListSecretsRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.ListSecretsRequest",secret_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        parent_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        page_size: "builtins.int|None|unset.UnsetType" = unset.Unset,
        page_token: "builtins.str|None|unset.UnsetType" = unset.Unset,
        show_scheduled_for_deletion: "builtins.bool|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(parent_id, unset.UnsetType):
            self.parent_id = parent_id
        if not isinstance(page_size, unset.UnsetType):
            self.page_size = page_size
        if not isinstance(page_token, unset.UnsetType):
            self.page_token = page_token
        if not isinstance(show_scheduled_for_deletion, unset.UnsetType):
            self.show_scheduled_for_deletion = show_scheduled_for_deletion
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "parent_id",
            "page_size",
            "page_token",
            "show_scheduled_for_deletion",
        ]
    
    @builtins.property
    def parent_id(self) -> "builtins.str":
        """
        ParentId of the secret to return.
        """
        
        return super()._get_field("parent_id", explicit_presence=False,
        )
    @parent_id.setter
    def parent_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("parent_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def page_size(self) -> "builtins.int":
        """
        The maximum number of results per page to return. If the number of available
        results is larger than [page_size], the service returns a [ListSecretsResponse.next_page_token]
        that can be used to get the next page of results in subsequent list requests.
        Default value: 100.
        """
        
        return super()._get_field("page_size", explicit_presence=False,
        )
    @page_size.setter
    def page_size(self, value: "builtins.int|None") -> None:
        return super()._set_field("page_size",value,explicit_presence=False,
        )
    
    @builtins.property
    def page_token(self) -> "builtins.str":
        """
        Page token. To get the next page of results, set [page_token] to the
        [ListSecretsResponse.next_page_token] returned by a previous list request.
        """
        
        return super()._get_field("page_token", explicit_presence=False,
        )
    @page_token.setter
    def page_token(self, value: "builtins.str|None") -> None:
        return super()._set_field("page_token",value,explicit_presence=False,
        )
    
    @builtins.property
    def show_scheduled_for_deletion(self) -> "builtins.bool":
        """
        By default, List operation doesn't include resources that are scheduled for deletion.
        If show_scheduled_for_deletion = true, the listing includes resources that are scheduled for deletion.
        """
        
        return super()._get_field("show_scheduled_for_deletion", explicit_presence=False,
        )
    @show_scheduled_for_deletion.setter
    def show_scheduled_for_deletion(self, value: "builtins.bool|None") -> None:
        return super()._set_field("show_scheduled_for_deletion",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "parent_id":"parent_id",
        "page_size":"page_size",
        "page_token":"page_token",
        "show_scheduled_for_deletion":"show_scheduled_for_deletion",
    }
    
class ListSecretsResponse(pb_classes.Message):
    __PB2_CLASS__ = secret_service_pb2.ListSecretsResponse
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.ListSecretsResponse",secret_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        next_page_token: "builtins.str|None|unset.UnsetType" = unset.Unset,
        items: "abc.Iterable[Secret]|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(next_page_token, unset.UnsetType):
            self.next_page_token = next_page_token
        if not isinstance(items, unset.UnsetType):
            self.items = items
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "next_page_token",
            "items",
        ]
    
    @builtins.property
    def next_page_token(self) -> "builtins.str":
        """
        This token allows you to get the next page of results for list requests. If the number
        of results is greater than the specified [ListSecretsRequest.page_size], use
        the [next_page_token] as the value for the [ListSecretsRequest.page_token] query parameter
        in the next list request. Each subsequent list request will have its own
        [next_page_token] to continue paging through the results.
        """
        
        return super()._get_field("next_page_token", explicit_presence=False,
        )
    @next_page_token.setter
    def next_page_token(self, value: "builtins.str|None") -> None:
        return super()._set_field("next_page_token",value,explicit_presence=False,
        )
    
    @builtins.property
    def items(self) -> "abc.MutableSequence[Secret]":
        """
        List of secrets in the specified container, without its Payload.
        """
        
        return super()._get_field("items", explicit_presence=False,
        wrap=pb_classes.Repeated.with_wrap(Secret,None,None),
        )
    @items.setter
    def items(self, value: "abc.Iterable[Secret]|None") -> None:
        return super()._set_field("items",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "next_page_token":"next_page_token",
        "items":"items",
    }
    
class DeleteSecretRequest(pb_classes.Message):
    __PB2_CLASS__ = secret_service_pb2.DeleteSecretRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.DeleteSecretRequest",secret_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        """
        ID of the secret to soft-delete.
        """
        
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
    }
    
class UndeleteSecretRequest(pb_classes.Message):
    __PB2_CLASS__ = secret_service_pb2.UndeleteSecretRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.UndeleteSecretRequest",secret_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
        if not isinstance(name, unset.UnsetType):
            self.name = name
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
            "name",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        """
        ID of the secret version to soft-delete.
        """
        
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    @builtins.property
    def name(self) -> "builtins.str":
        """
        A new Secret name in case the current one is already in use.
        Empty value in name field means "use original name for undeleted Secret".
        Undelete will fail if a Secret with the same name already exists.
        """
        
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
        "name":"name",
    }
    

class SecretServiceClient(client.ClientWithOperations[v1_1.Operation,v1_1.OperationServiceClient]):
    """
    A set of methods for managing secrets.
    
    This class provides the client methods for the ``.nebius.mysterybox.v1.SecretService`` service.
    
    Each method constructs a :class:`nebius.aio.request.Request` object
    that represents the in-flight RPC. The request can be awaited (async)
    or waited synchronously using its ``.wait()`` helpers.
    
    The request methods accept various parameters to configure metadata,
    timeouts, authorization, and retries. See individual method docstrings
    for details.
    
    :cvar __service_name__: The full protobuf service name.
    """
    
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.ServiceDescriptor](".nebius.mysterybox.v1.SecretService",secret_service_pb2.DESCRIPTOR,descriptor_1.ServiceDescriptor)
    """The protobuf service descriptor extraction function."""
    __service_name__ = ".nebius.mysterybox.v1.SecretService"
    __operation_type__ = v1_1.Operation
    __operation_service_class__ = v1_1.OperationServiceClient
    __operation_source_method__ = "Create"
    """The method name that can be used to fetch the address channel for the operation."""
    
    def create(self,
        request: "CreateSecretRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["CreateSecretRequest","operation.Operation[v1_1.Operation]"]:
        """
        Creates a new secret in the specified container.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.mysterybox.v1.CreateSecretRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.common.v1.Operation`.
        """
        
        return super().request(
            method="Create",
            request=request,
            result_pb2_class=operation_pb2.Operation,
            result_wrapper=operation.Operation,
            **kwargs,
        )
    
    def update(self,
        request: "UpdateSecretRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["UpdateSecretRequest","operation.Operation[v1_1.Operation]"]:
        """
        Updates an existing secret, excluded its Payload.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.mysterybox.v1.UpdateSecretRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.common.v1.Operation`.
        """
        
        kwargs['metadata'] = fieldmask_protobuf.ensure_reset_mask_in_metadata(request, kwargs.get('metadata', None))
        return super().request(
            method="Update",
            request=request,
            result_pb2_class=operation_pb2.Operation,
            result_wrapper=operation.Operation,
            **kwargs,
        )
    
    def get(self,
        request: "GetSecretRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["GetSecretRequest","Secret"]:
        """
        Returns the specified secret, without its Payload.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.mysterybox.v1.GetSecretRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.mysterybox.v1.Secret`.
        """
        
        return super().request(
            method="Get",
            request=request,
            result_pb2_class=secret_pb2.Secret,
            result_wrapper=pb_classes.simple_wrapper(Secret),
            **kwargs,
        )
    
    def get_by_name(self,
        request: "GetSecretByNameRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["GetSecretByNameRequest","Secret"]:
        """
        Returns the specified secret by name, without its Payload.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.mysterybox.v1.GetSecretByNameRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.mysterybox.v1.Secret`.
        """
        
        return super().request(
            method="GetByName",
            request=request,
            result_pb2_class=secret_pb2.Secret,
            result_wrapper=pb_classes.simple_wrapper(Secret),
            **kwargs,
        )
    
    def list(self,
        request: "ListSecretsRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["ListSecretsRequest","ListSecretsResponse"]:
        """
        Returns the list of secrets in a specified container, without its Payload.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.mysterybox.v1.ListSecretsRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.mysterybox.v1.ListSecretsResponse`.
        """
        
        return super().request(
            method="List",
            request=request,
            result_pb2_class=secret_service_pb2.ListSecretsResponse,
            result_wrapper=pb_classes.simple_wrapper(ListSecretsResponse),
            **kwargs,
        )
    
    def delete(self,
        request: "DeleteSecretRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["DeleteSecretRequest","operation.Operation[v1_1.Operation]"]:
        """
        Soft delete secret
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.mysterybox.v1.DeleteSecretRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.common.v1.Operation`.
        """
        
        return super().request(
            method="Delete",
            request=request,
            result_pb2_class=operation_pb2.Operation,
            result_wrapper=operation.Operation,
            **kwargs,
        )
    
    def undelete(self,
        request: "UndeleteSecretRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["UndeleteSecretRequest","operation.Operation[v1_1.Operation]"]:
        """
        Undelete secret version
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.mysterybox.v1.UndeleteSecretRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.common.v1.Operation`.
        """
        
        return super().request(
            method="Undelete",
            request=request,
            result_pb2_class=operation_pb2.Operation,
            result_wrapper=operation.Operation,
            **kwargs,
        )
    

# file: nebius/mysterybox/v1/secret_version_service.proto
class CreateSecretVersionRequest(pb_classes.Message):
    __PB2_CLASS__ = secret_version_service_pb2.CreateSecretVersionRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.CreateSecretVersionRequest",secret_version_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        metadata: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None|unset.UnsetType" = unset.Unset,
        spec: "SecretVersionSpec|secret_version_pb2.SecretVersionSpec|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(metadata, unset.UnsetType):
            self.metadata = metadata
        if not isinstance(spec, unset.UnsetType):
            self.spec = spec
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "metadata",
            "spec",
        ]
    
    @builtins.property
    def metadata(self) -> "v1_1.ResourceMetadata":
        return super()._get_field("metadata", explicit_presence=False,
        wrap=v1_1.ResourceMetadata,
        )
    @metadata.setter
    def metadata(self, value: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None") -> None:
        return super()._set_field("metadata",value,explicit_presence=False,
        )
    
    @builtins.property
    def spec(self) -> "SecretVersionSpec":
        return super()._get_field("spec", explicit_presence=False,
        wrap=SecretVersionSpec,
        )
    @spec.setter
    def spec(self, value: "SecretVersionSpec|secret_version_pb2.SecretVersionSpec|None") -> None:
        return super()._set_field("spec",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "metadata":"metadata",
        "spec":"spec",
    }
    
class GetSecretVersionRequest(pb_classes.Message):
    __PB2_CLASS__ = secret_version_service_pb2.GetSecretVersionRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.GetSecretVersionRequest",secret_version_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        show_scheduled_for_deletion: "builtins.bool|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
        if not isinstance(show_scheduled_for_deletion, unset.UnsetType):
            self.show_scheduled_for_deletion = show_scheduled_for_deletion
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
            "show_scheduled_for_deletion",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        """
        ID of secret versions to return.
        """
        
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    @builtins.property
    def show_scheduled_for_deletion(self) -> "builtins.bool":
        """
        By default, Get doesn't return resource if it is scheduled for deletion.
        If show_scheduled_for_deletion = true, the Get operation returns the resource even if it is scheduled for deletion.
        If show_scheduled_for_deletion = false, the Get method returns the NOT_FOUND gRPC status code.
        """
        
        return super()._get_field("show_scheduled_for_deletion", explicit_presence=False,
        )
    @show_scheduled_for_deletion.setter
    def show_scheduled_for_deletion(self, value: "builtins.bool|None") -> None:
        return super()._set_field("show_scheduled_for_deletion",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
        "show_scheduled_for_deletion":"show_scheduled_for_deletion",
    }
    
class ListSecretVersionsRequest(pb_classes.Message):
    __PB2_CLASS__ = secret_version_service_pb2.ListSecretVersionsRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.ListSecretVersionsRequest",secret_version_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        parent_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        page_size: "builtins.int|None|unset.UnsetType" = unset.Unset,
        page_token: "builtins.str|None|unset.UnsetType" = unset.Unset,
        show_scheduled_for_deletion: "builtins.bool|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(parent_id, unset.UnsetType):
            self.parent_id = parent_id
        if not isinstance(page_size, unset.UnsetType):
            self.page_size = page_size
        if not isinstance(page_token, unset.UnsetType):
            self.page_token = page_token
        if not isinstance(show_scheduled_for_deletion, unset.UnsetType):
            self.show_scheduled_for_deletion = show_scheduled_for_deletion
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "parent_id",
            "page_size",
            "page_token",
            "show_scheduled_for_deletion",
        ]
    
    @builtins.property
    def parent_id(self) -> "builtins.str":
        """
        Secret id related to the version.
        """
        
        return super()._get_field("parent_id", explicit_presence=False,
        )
    @parent_id.setter
    def parent_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("parent_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def page_size(self) -> "builtins.int":
        """
        The maximum number of results per page to return. If the number of available
        results is larger than [page_size], the service returns a [ListSecretVersionsResponse.next_page_token]
        that can be used to get the next page of results in subsequent list requests.
        Default value: 100.
        """
        
        return super()._get_field("page_size", explicit_presence=False,
        )
    @page_size.setter
    def page_size(self, value: "builtins.int|None") -> None:
        return super()._set_field("page_size",value,explicit_presence=False,
        )
    
    @builtins.property
    def page_token(self) -> "builtins.str":
        """
        Page token. To get the next page of results, set [page_token] to the
        [ListSecretVersionsResponse.next_page_token] returned by a previous list request.
        """
        
        return super()._get_field("page_token", explicit_presence=False,
        )
    @page_token.setter
    def page_token(self, value: "builtins.str|None") -> None:
        return super()._set_field("page_token",value,explicit_presence=False,
        )
    
    @builtins.property
    def show_scheduled_for_deletion(self) -> "builtins.bool":
        """
        By default, List operation doesn't include resources that are scheduled for deletion.
        If show_scheduled_for_deletion = true, the listing includes resources that are scheduled for deletion.
        """
        
        return super()._get_field("show_scheduled_for_deletion", explicit_presence=False,
        )
    @show_scheduled_for_deletion.setter
    def show_scheduled_for_deletion(self, value: "builtins.bool|None") -> None:
        return super()._set_field("show_scheduled_for_deletion",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "parent_id":"parent_id",
        "page_size":"page_size",
        "page_token":"page_token",
        "show_scheduled_for_deletion":"show_scheduled_for_deletion",
    }
    
class ListSecretVersionsResponse(pb_classes.Message):
    __PB2_CLASS__ = secret_version_service_pb2.ListSecretVersionsResponse
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.ListSecretVersionsResponse",secret_version_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        next_page_token: "builtins.str|None|unset.UnsetType" = unset.Unset,
        items: "abc.Iterable[SecretVersion]|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(next_page_token, unset.UnsetType):
            self.next_page_token = next_page_token
        if not isinstance(items, unset.UnsetType):
            self.items = items
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "next_page_token",
            "items",
        ]
    
    @builtins.property
    def next_page_token(self) -> "builtins.str":
        """
        This token allows you to get the next page of results for list requests. If the number
        of results is greater than the specified [ListSecretVersionsRequest.page_size], use
        the [next_page_token] as the value for the [ListSecretVersionsRequest.page_token] query parameter
        in the next list request. Each subsequent list request will have its own
        [next_page_token] to continue paging through the results.
        """
        
        return super()._get_field("next_page_token", explicit_presence=False,
        )
    @next_page_token.setter
    def next_page_token(self, value: "builtins.str|None") -> None:
        return super()._set_field("next_page_token",value,explicit_presence=False,
        )
    
    @builtins.property
    def items(self) -> "abc.MutableSequence[SecretVersion]":
        """
        List of secret versions in the specified container, without its Payload.
        """
        
        return super()._get_field("items", explicit_presence=False,
        wrap=pb_classes.Repeated.with_wrap(SecretVersion,None,None),
        )
    @items.setter
    def items(self, value: "abc.Iterable[SecretVersion]|None") -> None:
        return super()._set_field("items",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "next_page_token":"next_page_token",
        "items":"items",
    }
    
class DeleteSecretVersionRequest(pb_classes.Message):
    __PB2_CLASS__ = secret_version_service_pb2.DeleteSecretVersionRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.DeleteSecretVersionRequest",secret_version_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        """
        ID of the secret version to soft-delete.
        """
        
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
    }
    
class UndeleteSecretVersionRequest(pb_classes.Message):
    __PB2_CLASS__ = secret_version_service_pb2.UndeleteSecretVersionRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.mysterybox.v1.UndeleteSecretVersionRequest",secret_version_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        """
        ID of the secret version to soft-delete.
        """
        
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
    }
    

class SecretVersionServiceClient(client.ClientWithOperations[v1_1.Operation,v1_1.OperationServiceClient]):
    """
    A set of methods for managing secret's versions.
    
    This class provides the client methods for the ``.nebius.mysterybox.v1.SecretVersionService`` service.
    
    Each method constructs a :class:`nebius.aio.request.Request` object
    that represents the in-flight RPC. The request can be awaited (async)
    or waited synchronously using its ``.wait()`` helpers.
    
    The request methods accept various parameters to configure metadata,
    timeouts, authorization, and retries. See individual method docstrings
    for details.
    
    :cvar __service_name__: The full protobuf service name.
    """
    
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.ServiceDescriptor](".nebius.mysterybox.v1.SecretVersionService",secret_version_service_pb2.DESCRIPTOR,descriptor_1.ServiceDescriptor)
    """The protobuf service descriptor extraction function."""
    __service_name__ = ".nebius.mysterybox.v1.SecretVersionService"
    __operation_type__ = v1_1.Operation
    __operation_service_class__ = v1_1.OperationServiceClient
    __operation_source_method__ = "Create"
    """The method name that can be used to fetch the address channel for the operation."""
    
    def create(self,
        request: "CreateSecretVersionRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["CreateSecretVersionRequest","operation.Operation[v1_1.Operation]"]:
        """
        Creates a new secret version in the specified container.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.mysterybox.v1.CreateSecretVersionRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.common.v1.Operation`.
        """
        
        return super().request(
            method="Create",
            request=request,
            result_pb2_class=operation_pb2.Operation,
            result_wrapper=operation.Operation,
            **kwargs,
        )
    
    def get(self,
        request: "GetSecretVersionRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["GetSecretVersionRequest","SecretVersion"]:
        """
        Returns the specified secret version, without its Payload.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.mysterybox.v1.GetSecretVersionRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.mysterybox.v1.SecretVersion`.
        """
        
        return super().request(
            method="Get",
            request=request,
            result_pb2_class=secret_version_pb2.SecretVersion,
            result_wrapper=pb_classes.simple_wrapper(SecretVersion),
            **kwargs,
        )
    
    def list(self,
        request: "ListSecretVersionsRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["ListSecretVersionsRequest","ListSecretVersionsResponse"]:
        """
        Returns the list of secret versions in a specified container, without its Payload.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.mysterybox.v1.ListSecretVersionsRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.mysterybox.v1.ListSecretVersionsResponse`.
        """
        
        return super().request(
            method="List",
            request=request,
            result_pb2_class=secret_version_service_pb2.ListSecretVersionsResponse,
            result_wrapper=pb_classes.simple_wrapper(ListSecretVersionsResponse),
            **kwargs,
        )
    
    def delete(self,
        request: "DeleteSecretVersionRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["DeleteSecretVersionRequest","operation.Operation[v1_1.Operation]"]:
        """
        Soft delete secret version.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.mysterybox.v1.DeleteSecretVersionRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.common.v1.Operation`.
        """
        
        return super().request(
            method="Delete",
            request=request,
            result_pb2_class=operation_pb2.Operation,
            result_wrapper=operation.Operation,
            **kwargs,
        )
    
    def undelete(self,
        request: "UndeleteSecretVersionRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["UndeleteSecretVersionRequest","operation.Operation[v1_1.Operation]"]:
        """
        Undelete secret version
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.mysterybox.v1.UndeleteSecretVersionRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.common.v1.Operation`.
        """
        
        return super().request(
            method="Undelete",
            request=request,
            result_pb2_class=operation_pb2.Operation,
            result_wrapper=operation.Operation,
            **kwargs,
        )
    

__all__ = [
    #@ local import names here @#
    "Payload",
    "GetPayloadRequest",
    "GetPayloadByKeyRequest",
    "SecretPayload",
    "SecretPayloadEntry",
    "PayloadServiceClient",
    "SecretVersion",
    "SecretVersionSpec",
    "SecretVersionStatus",
    "Secret",
    "SecretSpec",
    "SecretStatus",
    "CreateSecretRequest",
    "UpdateSecretRequest",
    "GetSecretRequest",
    "GetSecretByNameRequest",
    "ListSecretsRequest",
    "ListSecretsResponse",
    "DeleteSecretRequest",
    "UndeleteSecretRequest",
    "SecretServiceClient",
    "CreateSecretVersionRequest",
    "GetSecretVersionRequest",
    "ListSecretVersionsRequest",
    "ListSecretVersionsResponse",
    "DeleteSecretVersionRequest",
    "UndeleteSecretVersionRequest",
    "SecretVersionServiceClient",
]



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Transfer(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: TransferSpec
    status: TransferStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[TransferSpec, _Mapping]] = ..., status: _Optional[_Union[TransferStatus, _Mapping]] = ...) -> None: ...

class TransferSpec(_message.Message):
    __slots__ = ["source", "destination", "after_one_iteration", "after_n_empty_iterations", "infinite", "inter_iteration_interval", "overwrite_strategy"]
    class OverwriteStrategy(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        OVERWRITE_STRATEGY_UNSPECIFIED: _ClassVar[TransferSpec.OverwriteStrategy]
        NEVER: _ClassVar[TransferSpec.OverwriteStrategy]
        IF_NEWER: _ClassVar[TransferSpec.OverwriteStrategy]
        ALWAYS: _ClassVar[TransferSpec.OverwriteStrategy]
    OVERWRITE_STRATEGY_UNSPECIFIED: TransferSpec.OverwriteStrategy
    NEVER: TransferSpec.OverwriteStrategy
    IF_NEWER: TransferSpec.OverwriteStrategy
    ALWAYS: TransferSpec.OverwriteStrategy
    class SourceBucket(_message.Message):
        __slots__ = ["endpoint", "bucket_name", "region", "credentials", "limiters"]
        ENDPOINT_FIELD_NUMBER: _ClassVar[int]
        BUCKET_NAME_FIELD_NUMBER: _ClassVar[int]
        REGION_FIELD_NUMBER: _ClassVar[int]
        CREDENTIALS_FIELD_NUMBER: _ClassVar[int]
        LIMITERS_FIELD_NUMBER: _ClassVar[int]
        endpoint: str
        bucket_name: str
        region: str
        credentials: TransferSpec.BucketCredentials
        limiters: TransferSpec.Limiters
        def __init__(self, endpoint: _Optional[str] = ..., bucket_name: _Optional[str] = ..., region: _Optional[str] = ..., credentials: _Optional[_Union[TransferSpec.BucketCredentials, _Mapping]] = ..., limiters: _Optional[_Union[TransferSpec.Limiters, _Mapping]] = ...) -> None: ...
    class DestinationBucket(_message.Message):
        __slots__ = ["bucket_name", "credentials"]
        BUCKET_NAME_FIELD_NUMBER: _ClassVar[int]
        CREDENTIALS_FIELD_NUMBER: _ClassVar[int]
        bucket_name: str
        credentials: TransferSpec.BucketCredentials
        def __init__(self, bucket_name: _Optional[str] = ..., credentials: _Optional[_Union[TransferSpec.BucketCredentials, _Mapping]] = ...) -> None: ...
    class BucketCredentials(_message.Message):
        __slots__ = ["anonymous", "access_key", "azure_access_key"]
        class CredentialsAnonymous(_message.Message):
            __slots__ = []
            def __init__(self) -> None: ...
        class CredentialsAccessKey(_message.Message):
            __slots__ = ["access_key_id", "secret_access_key"]
            ACCESS_KEY_ID_FIELD_NUMBER: _ClassVar[int]
            SECRET_ACCESS_KEY_FIELD_NUMBER: _ClassVar[int]
            access_key_id: str
            secret_access_key: str
            def __init__(self, access_key_id: _Optional[str] = ..., secret_access_key: _Optional[str] = ...) -> None: ...
        class AzureAccessKey(_message.Message):
            __slots__ = ["account_name", "access_key"]
            ACCOUNT_NAME_FIELD_NUMBER: _ClassVar[int]
            ACCESS_KEY_FIELD_NUMBER: _ClassVar[int]
            account_name: str
            access_key: str
            def __init__(self, account_name: _Optional[str] = ..., access_key: _Optional[str] = ...) -> None: ...
        ANONYMOUS_FIELD_NUMBER: _ClassVar[int]
        ACCESS_KEY_FIELD_NUMBER: _ClassVar[int]
        AZURE_ACCESS_KEY_FIELD_NUMBER: _ClassVar[int]
        anonymous: TransferSpec.BucketCredentials.CredentialsAnonymous
        access_key: TransferSpec.BucketCredentials.CredentialsAccessKey
        azure_access_key: TransferSpec.BucketCredentials.AzureAccessKey
        def __init__(self, anonymous: _Optional[_Union[TransferSpec.BucketCredentials.CredentialsAnonymous, _Mapping]] = ..., access_key: _Optional[_Union[TransferSpec.BucketCredentials.CredentialsAccessKey, _Mapping]] = ..., azure_access_key: _Optional[_Union[TransferSpec.BucketCredentials.AzureAccessKey, _Mapping]] = ...) -> None: ...
    class Limiters(_message.Message):
        __slots__ = ["bandwidth_bytes_per_second", "requests_per_second"]
        BANDWIDTH_BYTES_PER_SECOND_FIELD_NUMBER: _ClassVar[int]
        REQUESTS_PER_SECOND_FIELD_NUMBER: _ClassVar[int]
        bandwidth_bytes_per_second: int
        requests_per_second: int
        def __init__(self, bandwidth_bytes_per_second: _Optional[int] = ..., requests_per_second: _Optional[int] = ...) -> None: ...
    class StopConditionAfterOneIteration(_message.Message):
        __slots__ = []
        def __init__(self) -> None: ...
    class StopConditionAfterNEmptyIterations(_message.Message):
        __slots__ = ["empty_iterations_threshold"]
        EMPTY_ITERATIONS_THRESHOLD_FIELD_NUMBER: _ClassVar[int]
        empty_iterations_threshold: int
        def __init__(self, empty_iterations_threshold: _Optional[int] = ...) -> None: ...
    class StopConditionInfinite(_message.Message):
        __slots__ = []
        def __init__(self) -> None: ...
    SOURCE_FIELD_NUMBER: _ClassVar[int]
    DESTINATION_FIELD_NUMBER: _ClassVar[int]
    AFTER_ONE_ITERATION_FIELD_NUMBER: _ClassVar[int]
    AFTER_N_EMPTY_ITERATIONS_FIELD_NUMBER: _ClassVar[int]
    INFINITE_FIELD_NUMBER: _ClassVar[int]
    INTER_ITERATION_INTERVAL_FIELD_NUMBER: _ClassVar[int]
    OVERWRITE_STRATEGY_FIELD_NUMBER: _ClassVar[int]
    source: TransferSpec.SourceBucket
    destination: TransferSpec.DestinationBucket
    after_one_iteration: TransferSpec.StopConditionAfterOneIteration
    after_n_empty_iterations: TransferSpec.StopConditionAfterNEmptyIterations
    infinite: TransferSpec.StopConditionInfinite
    inter_iteration_interval: _duration_pb2.Duration
    overwrite_strategy: TransferSpec.OverwriteStrategy
    def __init__(self, source: _Optional[_Union[TransferSpec.SourceBucket, _Mapping]] = ..., destination: _Optional[_Union[TransferSpec.DestinationBucket, _Mapping]] = ..., after_one_iteration: _Optional[_Union[TransferSpec.StopConditionAfterOneIteration, _Mapping]] = ..., after_n_empty_iterations: _Optional[_Union[TransferSpec.StopConditionAfterNEmptyIterations, _Mapping]] = ..., infinite: _Optional[_Union[TransferSpec.StopConditionInfinite, _Mapping]] = ..., inter_iteration_interval: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., overwrite_strategy: _Optional[_Union[TransferSpec.OverwriteStrategy, str]] = ...) -> None: ...

class TransferStatus(_message.Message):
    __slots__ = ["state", "error", "suspension_state", "last_iteration"]
    class State(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        STATE_UNSPECIFIED: _ClassVar[TransferStatus.State]
        ACTIVE: _ClassVar[TransferStatus.State]
        STOPPING: _ClassVar[TransferStatus.State]
        STOPPED: _ClassVar[TransferStatus.State]
        FAILING: _ClassVar[TransferStatus.State]
        FAILED: _ClassVar[TransferStatus.State]
    STATE_UNSPECIFIED: TransferStatus.State
    ACTIVE: TransferStatus.State
    STOPPING: TransferStatus.State
    STOPPED: TransferStatus.State
    FAILING: TransferStatus.State
    FAILED: TransferStatus.State
    class SuspensionState(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        SUSPENSION_STATE_UNSPECIFIED: _ClassVar[TransferStatus.SuspensionState]
        NOT_SUSPENDED: _ClassVar[TransferStatus.SuspensionState]
        SUSPENDED: _ClassVar[TransferStatus.SuspensionState]
    SUSPENSION_STATE_UNSPECIFIED: TransferStatus.SuspensionState
    NOT_SUSPENDED: TransferStatus.SuspensionState
    SUSPENDED: TransferStatus.SuspensionState
    STATE_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    SUSPENSION_STATE_FIELD_NUMBER: _ClassVar[int]
    LAST_ITERATION_FIELD_NUMBER: _ClassVar[int]
    state: TransferStatus.State
    error: str
    suspension_state: TransferStatus.SuspensionState
    last_iteration: TransferIteration
    def __init__(self, state: _Optional[_Union[TransferStatus.State, str]] = ..., error: _Optional[str] = ..., suspension_state: _Optional[_Union[TransferStatus.SuspensionState, str]] = ..., last_iteration: _Optional[_Union[TransferIteration, _Mapping]] = ...) -> None: ...

class TransferIteration(_message.Message):
    __slots__ = ["sequence_number", "state", "error", "start_time", "end_time", "objects_discovered_count", "objects_migrated_count", "objects_skipped_count", "objects_migrated_size", "average_throughput_bytes"]
    class State(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        STATE_UNSPECIFIED: _ClassVar[TransferIteration.State]
        IN_PROGRESS: _ClassVar[TransferIteration.State]
        COMPLETED: _ClassVar[TransferIteration.State]
        STOPPED: _ClassVar[TransferIteration.State]
        FAILED: _ClassVar[TransferIteration.State]
    STATE_UNSPECIFIED: TransferIteration.State
    IN_PROGRESS: TransferIteration.State
    COMPLETED: TransferIteration.State
    STOPPED: TransferIteration.State
    FAILED: TransferIteration.State
    SEQUENCE_NUMBER_FIELD_NUMBER: _ClassVar[int]
    STATE_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    START_TIME_FIELD_NUMBER: _ClassVar[int]
    END_TIME_FIELD_NUMBER: _ClassVar[int]
    OBJECTS_DISCOVERED_COUNT_FIELD_NUMBER: _ClassVar[int]
    OBJECTS_MIGRATED_COUNT_FIELD_NUMBER: _ClassVar[int]
    OBJECTS_SKIPPED_COUNT_FIELD_NUMBER: _ClassVar[int]
    OBJECTS_MIGRATED_SIZE_FIELD_NUMBER: _ClassVar[int]
    AVERAGE_THROUGHPUT_BYTES_FIELD_NUMBER: _ClassVar[int]
    sequence_number: int
    state: TransferIteration.State
    error: str
    start_time: _timestamp_pb2.Timestamp
    end_time: _timestamp_pb2.Timestamp
    objects_discovered_count: int
    objects_migrated_count: int
    objects_skipped_count: int
    objects_migrated_size: int
    average_throughput_bytes: int
    def __init__(self, sequence_number: _Optional[int] = ..., state: _Optional[_Union[TransferIteration.State, str]] = ..., error: _Optional[str] = ..., start_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., end_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., objects_discovered_count: _Optional[int] = ..., objects_migrated_count: _Optional[int] = ..., objects_skipped_count: _Optional[int] = ..., objects_migrated_size: _Optional[int] = ..., average_throughput_bytes: _Optional[int] = ...) -> None: ...



from collections.abc import Iterable, MutableSequence, Sequence
from typing import overload


class Authorization:
    DISABLE = "disable"


class Metadata(MutableSequence[tuple[str, str]]):
    def __init__(self, initial: Iterable[tuple[str, str]] | None = None) -> None:
        self._contents = list[tuple[str, str]]()
        if initial is not None:
            for k, v in initial:
                if isinstance(k, str) and isinstance(v, str):  # type: ignore[unused-ignore]
                    self._contents.append((k.lower(), v))

    def insert(self, index: int, value: tuple[str, str]) -> None:
        self._contents.insert(index, (value[0].lower(), value[1]))

    @overload
    def get_one(
        self,
        index: str,
        default: str,
        first: bool = False,
    ) -> str: ...

    @overload
    def get_one(
        self,
        index: str,
        default: None = None,
        first: bool = False,
    ) -> str | None: ...

    def get_one(
        self,
        index: str,
        default: str | None = None,
        first: bool = False,
    ) -> str | None:
        try:
            return self[index][0 if first else -1]
        except IndexError:
            return default

    def add(
        self,
        index: str,
        value: str,
    ) -> None:
        self._contents.append((index.lower(), value))

    @overload
    def __getitem__(self, index: int) -> tuple[str, str]: ...

    @overload
    def __getitem__(self, index: slice) -> MutableSequence[tuple[str, str]]: ...

    @overload
    def __getitem__(self, index: str) -> Sequence[str]: ...

    def __getitem__(
        self, index: int | slice | str
    ) -> tuple[str, str] | MutableSequence[tuple[str, str]] | Sequence[str]:
        if isinstance(index, int) or isinstance(index, slice):
            return self._contents[index]
        if isinstance(index, str):  # type: ignore[unused-ignore]
            index = index.lower()
            return [v for k, v in self._contents if k == index]
        raise TypeError("Index must be int, str or slice")

    def __has__(self, key: str) -> bool:
        key = key.lower()
        for k, _ in self._contents:
            if k == key:
                return True
        return False

    def __setitem__(
        self,
        index: int | slice | str,
        value: tuple[str, str] | Iterable[tuple[str, str]] | Iterable[str] | str,
    ) -> None:
        if isinstance(index, int):
            if (
                isinstance(value, tuple)
                and len(value) == 2
                and isinstance(value[0], str)
                and isinstance(value[1], str)
            ):
                self._contents[index] = (value[0].lower(), value[1])
                return
            else:
                TypeError("If index is int, value must be Tuple[str,str]")
        if isinstance(index, slice):
            for i, v in zip(range(len(self))[index], value):  # type: ignore[unused-ignore]
                self[i] = v
            return
        if isinstance(index, str):  # type: ignore[unused-ignore]
            index = index.lower()
            del self[index]
            if isinstance(value, str):
                value = [value]
            for s in value:
                if isinstance(s, str):
                    self.append((index, s))
                else:
                    raise TypeError(
                        "If index is str, value must be str or Iterable[str]"
                    )
            return
        raise TypeError("Index must be int, str or slice")

    def __delitem__(self, index: int | slice | str) -> None:
        if isinstance(index, int) or isinstance(index, slice):
            del self._contents[index]
            return
        if isinstance(index, str):  # type: ignore[unused-ignore]
            index = index.lower()
            self._contents = [v for v in self._contents if v[0] != index]
            return
        raise TypeError("Index must be int, str or slice")

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}{list(self)}"

    def __len__(self) -> int:
        return len(self._contents)



"""A renewable file-backed bearer.

This module composes a wrapped network :class:`nebius.aio.token.token.Bearer` with a
:class:`ThrottledTokenCache` to provide cached tokens that are refreshed from
the wrapped bearer when the cached token is close to expiration.

Classes
-------

- :class:`RenewableFileCacheReceiver` -- A receiver that prefers the cached
  token when it's fresh, otherwise fetches a new token from the wrapped
  bearer and stores it in the cache.
- :class:`RenewableFileCacheBearer` -- A bearer that composes an existing
  :class:`nebius.aio.token.token.Bearer` instance with a file-backed cache and exposes
  the renewable receiver.

"""

from datetime import datetime, timedelta, timezone
from logging import getLogger
from pathlib import Path

from nebius.aio.token.token import Bearer as ParentBearer
from nebius.aio.token.token import Receiver as ParentReceiver
from nebius.aio.token.token import Token
from nebius.base.constants import DEFAULT_CONFIG_DIR, DEFAULT_CREDENTIALS_FILE

from .throttled_token_cache import ThrottledTokenCache

log = getLogger(__name__)


class RenewableFileCacheReceiver(ParentReceiver):
    """Receiver that returns a cached token when safe, or refreshes it.

    The receiver first consults the provided :class:`ThrottledTokenCache`.
    If the cached token is missing or too close to expiration (depending
    on the parent's ``safety_margin``), the receiver will fetch a fresh
    token from the wrapped bearer and persist it to the cache.

    :param bearer: The parent :class:`RenewableFileCacheBearer` containing
        configuration such as :attr:`safety_margin` and the wrapped bearer instance.
    :param cache: The throttled file-backed token cache.

    Behavior
    --------
    - When the cache holds a non-expired token and it is outside the
      safety margin, that token is returned immediately.
    - Otherwise a receiver from the wrapped bearer is used to fetch a
      fresh token. The new token is saved to the cache on success.

    :param bearer: The owning renewable bearer.
    :param cache: The throttled token cache used for storage.
    """

    def __init__(
        self,
        bearer: "RenewableFileCacheBearer",
        cache: ThrottledTokenCache,
    ) -> None:
        """Create a renewable receiver.

        No file I/O occurs during construction; the cache performs lazy
        I/O when accessed.
        """
        super().__init__()
        self._bearer = bearer
        self._cache = cache
        self._receiver: ParentReceiver | None = None
        self._last_saved: Token | None = None
        self._from_cache: bool = True

    async def _fetch(
        self, timeout: float | None = None, options: dict[str, str] | None = None
    ) -> Token:
        """Return a cached token when valid or fetch and store a new one.

        :param timeout: Pass-through timeout to the wrapped receiver.
        :param options: Optional fetch options forwarded to the wrapped
            receiver.
        :returns: A valid :class:`Token` (possibly empty).
        """
        if self._from_cache:
            token = await self._cache.get()
        else:
            token = await self._cache.refresh()
            if self._last_saved == token:
                # Avoid reusing the same token after an error; treat as
                # missing so the wrapped receiver will be invoked.
                token = None

        if token is not None and not token.is_expired():
            if self._bearer.safety_margin is None or (
                not token.expiration
                or (
                    token.expiration - self._bearer.safety_margin
                    > datetime.now(timezone.utc)
                )
            ):
                self._from_cache = True
                self._last_saved = token
                return token

        # Cache miss or token too close to expiry: fetch from wrapped
        # bearer.
        self._from_cache = False
        log.debug("Fetching new token from bearer")
        if self._receiver is None:
            # The wrapped bearer must exist; leave the type-ignore where
            # the static checker cannot prove it.
            self._receiver = self._bearer.wrapped.receiver()  # type: ignore

        token = await self._receiver.fetch(timeout=timeout, options=options)
        if token.is_empty():
            self._last_saved = None
            return token
        await self._cache.set(token)
        self._last_saved = token
        return token

    def can_retry(
        self,
        err: Exception,
        options: dict[str, str] | None = None,
    ) -> bool:
        """Decide whether an error should be retried.

        - If the previous attempt read from cache, allow one retry so
          callers can fall back to the wrapped receiver.
        - If no wrapped receiver exists yet, permit retry so the first
          fetch attempt can create it.
        - Otherwise defer to the wrapped receiver's ``can_retry``.

        :param err: The exception that occurred.
        :param options: Optional receiver options forwarded to the wrapped
            receiver when applicable.
        :returns: Whether the operation should be retried.
        """
        if self._from_cache:
            self._from_cache = False
            return True

        if self._receiver is None:
            return True  # Retry if we don't have a receiver yet

        return self._receiver.can_retry(err, options)


class RenewableFileCacheBearer(ParentBearer):
    """Bearer that composes a wrapped bearer with a file-backed cache.

    :ivar safety_margin: A :class:`datetime.timedelta` or seconds value
        used to determine when a token is considered too close to expiry.
        If ``None``, the cache will be used until tokens are expired.

    :param bearer: Wrapped bearer used for refreshing tokens. Must be named.
    :param safety_margin: Safety margin before token expiration.
    :param cache_file: Path to the file used for persistent cache.
    :param throttle: In-memory throttle interval for cache reads.

    Example
    -------

    Wrap a custom bearer with a name and file cache::

        from nebius.sdk import SDK
        from nebius.aio.token.token import Bearer, Receiver, Token
        from nebius.aio.token.file_cache.renewable_bearer import (
            RenewableFileCacheBearer
        )

        class SomeCustomHeavyLoadBearer(Bearer):
            def receiver(self) -> Receiver:
                return SomeReceiver()

        class SomeReceiver(Receiver):
            async def _fetch(self, timeout=None, options=None) -> Token:
                # Simulate heavy load token fetch
                return Token("heavy-token")

            def can_retry(self, err, options=None) -> bool:
                return False

        custom_bearer = SomeCustomHeavyLoadBearer()
        cached_bearer = RenewableFileCacheBearer(custom_bearer)

        sdk = SDK(credentials=cached_bearer)
    """

    def __init__(
        self,
        bearer: ParentBearer,
        safety_margin: timedelta | float = timedelta(hours=2),
        cache_file: str | Path = Path(DEFAULT_CONFIG_DIR) / DEFAULT_CREDENTIALS_FILE,
        throttle: timedelta | float = timedelta(minutes=5),
    ) -> None:
        """Create a renewable file-backed bearer.

        The constructor creates the underlying :class:`ThrottledTokenCache`.

        :raises ValueError: When the wrapped bearer has no name.
        """
        self._bearer = bearer
        if isinstance(safety_margin, (float, int)):
            safety_margin = timedelta(seconds=safety_margin)
        self.safety_margin: timedelta | None = safety_margin
        name = self._bearer.name
        if name is None:
            raise ValueError("Bearer must have a name for the cache.")
        self._cache = ThrottledTokenCache(
            name=name,
            cache_file=cache_file,
            throttle=throttle,
        )

    @property
    def wrapped(self) -> ParentBearer | None:
        """Return the wrapped bearer instance.

        :returns: The wrapped bearer passed at construction.
        """
        return self._bearer

    def receiver(self) -> ParentReceiver:
        """Return a :class:`RenewableFileCacheReceiver` bound to the cache.

        :returns: A receiver that uses the file cache and refreshes from
            the wrapped bearer when necessary.
        """
        return RenewableFileCacheReceiver(self, self._cache)



from abc import ABC
from typing import Generic, TypeVar

import google.protobuf.descriptor as pb

# Define the TypeVar for supported descriptor types
T = TypeVar(
    "T", pb.EnumDescriptor, pb.Descriptor, pb.OneofDescriptor, pb.ServiceDescriptor
)


class DescriptorWrap(ABC, Generic[T]):
    def __init__(
        self,
        name: str,
        file_descriptor: pb.FileDescriptor,
        expected_type: type[T],
    ) -> None:
        if name[0] == ".":
            name = name[1:]
        self._name = name
        self._file_descriptor = file_descriptor
        self._expected_type: type[T] = expected_type
        self._descriptor: T | None = None

    def __call__(self) -> T:
        """Retrieve the descriptor of the specified type using the fully qualified
        name."""
        if self._descriptor is not None:
            return self._descriptor
        descriptor = self._find_descriptor(self._file_descriptor, self._name)
        if descriptor is None:
            raise ValueError(f"No descriptor found for name {self._name}")
        if not isinstance(descriptor, self._expected_type):
            raise TypeError(
                f"Descriptor {self._name} is of type {type(descriptor).__name__}, "
                f"expected {self._expected_type.__name__}"
            )
        self._descriptor = descriptor
        return descriptor

    def _find_descriptor(
        self, container: pb.FileDescriptor | pb.Descriptor, name: str
    ) -> (
        pb.Descriptor
        | pb.EnumDescriptor
        | pb.OneofDescriptor
        | pb.ServiceDescriptor
        | None
    ):
        """
        Recursively searches for the descriptor in the given container (file or message)
        """
        # Check for top-level messages
        if isinstance(container, pb.FileDescriptor):
            for srv in container.services_by_name.values():
                if srv.full_name == name:
                    if not isinstance(srv, pb.ServiceDescriptor):
                        raise ValueError(f"Pool returned unexpected type {type(srv)}")
                    return srv
            for enum in container.enum_types_by_name.values():
                if enum.full_name == name:
                    if not isinstance(enum, pb.EnumDescriptor):
                        raise ValueError(f"Pool returned unexpected type {type(enum)}")
                    return enum
            for message in container.message_types_by_name.values():
                found = self._find_descriptor(message, name)
                if found:
                    return found

        # Check for nested messages and enums in a message
        else:
            if container.full_name == name:
                return container
            for nested_message in container.nested_types:
                found = self._find_descriptor(nested_message, name)
                if found:
                    return found
            for nested_enum in container.enum_types:
                if nested_enum.full_name == name:
                    if not isinstance(nested_enum, pb.EnumDescriptor):
                        raise ValueError(
                            f"Pool returned unexpected type {type(nested_enum)}"
                        )
                    return nested_enum
            for oneof in container.oneofs:
                if oneof.full_name == name:
                    if not isinstance(oneof, pb.OneofDescriptor):
                        raise ValueError(f"Pool returned unexpected type {type(oneof)}")
                    return oneof

        return None



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Tenant(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: TenantSpec
    status: TenantStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[TenantSpec, _Mapping]] = ..., status: _Optional[_Union[TenantStatus, _Mapping]] = ...) -> None: ...

class TenantSpec(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class TenantStatus(_message.Message):
    __slots__ = ["tenant_state"]
    class TenantState(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        STATE_UNSPECIFIED: _ClassVar[TenantStatus.TenantState]
        CREATING: _ClassVar[TenantStatus.TenantState]
        ACTIVE: _ClassVar[TenantStatus.TenantState]
        CREATED: _ClassVar[TenantStatus.TenantState]
        ACTIVATING: _ClassVar[TenantStatus.TenantState]
        PARKING: _ClassVar[TenantStatus.TenantState]
        PARKED: _ClassVar[TenantStatus.TenantState]
    STATE_UNSPECIFIED: TenantStatus.TenantState
    CREATING: TenantStatus.TenantState
    ACTIVE: TenantStatus.TenantState
    CREATED: TenantStatus.TenantState
    ACTIVATING: TenantStatus.TenantState
    PARKING: TenantStatus.TenantState
    PARKED: TenantStatus.TenantState
    TENANT_STATE_FIELD_NUMBER: _ClassVar[int]
    tenant_state: TenantStatus.TenantState
    def __init__(self, tenant_state: _Optional[_Union[TenantStatus.TenantState, str]] = ...) -> None: ...



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.common.v1 import operation_pb2 as nebius_dot_common_dot_v1_dot_operation__pb2
from nebius.api.nebius.storage.v1alpha1 import transfer_pb2 as nebius_dot_storage_dot_v1alpha1_dot_transfer__pb2
from nebius.api.nebius.storage.v1alpha1 import transfer_service_pb2 as nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2


class TransferServiceStub(object):
    """TransferService provides functionality for creating and managing transfers
    that migrate data from other S3 providers or across different regions of Nebius Object Storage.
    Each transfer consists of consecutive iterations where the service lists objects in the source bucket and
    moves those that need to be transferred according to the specified overwrite strategy.
    After an iteration completes, the transfer will stop if its stop condition is met. Otherwise,
    it will wait for the defined inter-iteration interval before starting the next iteration.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Get = channel.unary_unary(
                '/nebius.storage.v1alpha1.TransferService/Get',
                request_serializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.GetTransferRequest.SerializeToString,
                response_deserializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__pb2.Transfer.FromString,
                )
        self.GetByName = channel.unary_unary(
                '/nebius.storage.v1alpha1.TransferService/GetByName',
                request_serializer=nebius_dot_common_dot_v1_dot_metadata__pb2.GetByNameRequest.SerializeToString,
                response_deserializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__pb2.Transfer.FromString,
                )
        self.List = channel.unary_unary(
                '/nebius.storage.v1alpha1.TransferService/List',
                request_serializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.ListTransfersRequest.SerializeToString,
                response_deserializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.ListTransfersResponse.FromString,
                )
        self.Create = channel.unary_unary(
                '/nebius.storage.v1alpha1.TransferService/Create',
                request_serializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.CreateTransferRequest.SerializeToString,
                response_deserializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
                )
        self.Update = channel.unary_unary(
                '/nebius.storage.v1alpha1.TransferService/Update',
                request_serializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.UpdateTransferRequest.SerializeToString,
                response_deserializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
                )
        self.Stop = channel.unary_unary(
                '/nebius.storage.v1alpha1.TransferService/Stop',
                request_serializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.StopTransferRequest.SerializeToString,
                response_deserializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
                )
        self.Resume = channel.unary_unary(
                '/nebius.storage.v1alpha1.TransferService/Resume',
                request_serializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.ResumeTransferRequest.SerializeToString,
                response_deserializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
                )
        self.Delete = channel.unary_unary(
                '/nebius.storage.v1alpha1.TransferService/Delete',
                request_serializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.DeleteTransferRequest.SerializeToString,
                response_deserializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
                )
        self.GetIterationHistory = channel.unary_unary(
                '/nebius.storage.v1alpha1.TransferService/GetIterationHistory',
                request_serializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.GetIterationHistoryRequest.SerializeToString,
                response_deserializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.GetIterationHistoryResponse.FromString,
                )


class TransferServiceServicer(object):
    """TransferService provides functionality for creating and managing transfers
    that migrate data from other S3 providers or across different regions of Nebius Object Storage.
    Each transfer consists of consecutive iterations where the service lists objects in the source bucket and
    moves those that need to be transferred according to the specified overwrite strategy.
    After an iteration completes, the transfer will stop if its stop condition is met. Otherwise,
    it will wait for the defined inter-iteration interval before starting the next iteration.
    """

    def Get(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetByName(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def List(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Create(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Update(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Stop(self, request, context):
        """Stop active transfer. If the transfer is currently moving data,
        it will be transitioned to the Stopping state and will move to the Stopped state once all processes have been stopped.
        If the transfer is waiting between iterations, it will stop immediately.
        Only transfer in Active state can be stopped.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Resume(self, request, context):
        """Resume stopped or failed transfer.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Delete(self, request, context):
        """Fully delete a transfer along with its history. Only stopped or failed transfers can be deleted.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetIterationHistory(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_TransferServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Get': grpc.unary_unary_rpc_method_handler(
                    servicer.Get,
                    request_deserializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.GetTransferRequest.FromString,
                    response_serializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__pb2.Transfer.SerializeToString,
            ),
            'GetByName': grpc.unary_unary_rpc_method_handler(
                    servicer.GetByName,
                    request_deserializer=nebius_dot_common_dot_v1_dot_metadata__pb2.GetByNameRequest.FromString,
                    response_serializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__pb2.Transfer.SerializeToString,
            ),
            'List': grpc.unary_unary_rpc_method_handler(
                    servicer.List,
                    request_deserializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.ListTransfersRequest.FromString,
                    response_serializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.ListTransfersResponse.SerializeToString,
            ),
            'Create': grpc.unary_unary_rpc_method_handler(
                    servicer.Create,
                    request_deserializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.CreateTransferRequest.FromString,
                    response_serializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.SerializeToString,
            ),
            'Update': grpc.unary_unary_rpc_method_handler(
                    servicer.Update,
                    request_deserializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.UpdateTransferRequest.FromString,
                    response_serializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.SerializeToString,
            ),
            'Stop': grpc.unary_unary_rpc_method_handler(
                    servicer.Stop,
                    request_deserializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.StopTransferRequest.FromString,
                    response_serializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.SerializeToString,
            ),
            'Resume': grpc.unary_unary_rpc_method_handler(
                    servicer.Resume,
                    request_deserializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.ResumeTransferRequest.FromString,
                    response_serializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.SerializeToString,
            ),
            'Delete': grpc.unary_unary_rpc_method_handler(
                    servicer.Delete,
                    request_deserializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.DeleteTransferRequest.FromString,
                    response_serializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.SerializeToString,
            ),
            'GetIterationHistory': grpc.unary_unary_rpc_method_handler(
                    servicer.GetIterationHistory,
                    request_deserializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.GetIterationHistoryRequest.FromString,
                    response_serializer=nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.GetIterationHistoryResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'nebius.storage.v1alpha1.TransferService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class TransferService(object):
    """TransferService provides functionality for creating and managing transfers
    that migrate data from other S3 providers or across different regions of Nebius Object Storage.
    Each transfer consists of consecutive iterations where the service lists objects in the source bucket and
    moves those that need to be transferred according to the specified overwrite strategy.
    After an iteration completes, the transfer will stop if its stop condition is met. Otherwise,
    it will wait for the defined inter-iteration interval before starting the next iteration.
    """

    @staticmethod
    def Get(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.storage.v1alpha1.TransferService/Get',
            nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.GetTransferRequest.SerializeToString,
            nebius_dot_storage_dot_v1alpha1_dot_transfer__pb2.Transfer.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetByName(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.storage.v1alpha1.TransferService/GetByName',
            nebius_dot_common_dot_v1_dot_metadata__pb2.GetByNameRequest.SerializeToString,
            nebius_dot_storage_dot_v1alpha1_dot_transfer__pb2.Transfer.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def List(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.storage.v1alpha1.TransferService/List',
            nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.ListTransfersRequest.SerializeToString,
            nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.ListTransfersResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Create(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.storage.v1alpha1.TransferService/Create',
            nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.CreateTransferRequest.SerializeToString,
            nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Update(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.storage.v1alpha1.TransferService/Update',
            nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.UpdateTransferRequest.SerializeToString,
            nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Stop(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.storage.v1alpha1.TransferService/Stop',
            nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.StopTransferRequest.SerializeToString,
            nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Resume(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.storage.v1alpha1.TransferService/Resume',
            nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.ResumeTransferRequest.SerializeToString,
            nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Delete(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.storage.v1alpha1.TransferService/Delete',
            nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.DeleteTransferRequest.SerializeToString,
            nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetIterationHistory(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.storage.v1alpha1.TransferService/GetIterationHistory',
            nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.GetIterationHistoryRequest.SerializeToString,
            nebius_dot_storage_dot_v1alpha1_dot_transfer__service__pb2.GetIterationHistoryResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



"""Token exchange bearer and receiver.

This module implements a bearer that exchanges a permanent service-account JWT
or similar credentials for a short-lived access token using the
``TokenExchangeService``. The exchange is performed via a gRPC call
and the response is converted into a :class:`Token` instance.

The primary classes are:

- :class:`Receiver` -- performs the token exchange request for a single
    fetch and exposes ``can_retry`` logic.
- :class:`Bearer` -- holds the channel and requester and constructs
    per-request receivers.

Examples
--------

Create a bearer with a preconfigured channel::

    from nebius.aio.token.exchangeable import Bearer
    bearer = Bearer(requester, channel=my_channel)
    token = await bearer.receiver().fetch()

"""

from collections.abc import Awaitable, Coroutine
from datetime import datetime, timedelta, timezone
from logging import getLogger
from typing import Any

from grpc.aio import AioRpcError
from grpc_status import rpc_status

from nebius.aio.abc import ClientChannelInterface
from nebius.aio.authorization.options import OPTION_TYPE, Types
from nebius.api.nebius.iam.v1 import CreateTokenResponse, TokenExchangeServiceClient
from nebius.base.error import SDKError
from nebius.base.metadata import Metadata as NebiusMetadata
from nebius.base.service_account.service_account import TokenRequester
from nebius.base.token_sanitizer import TokenSanitizer

from .deferred_channel import DeferredChannel
from .options import OPTION_MAX_RETRIES
from .token import Bearer as ParentBearer
from .token import Receiver as ParentReceiver
from .token import Token

sanitizer = TokenSanitizer.access_token_sanitizer()

log = getLogger(__name__)


class UnsupportedResponseError(SDKError):
    """Raised when the token exchange returned an unexpected response type.

    :param expected: The expected response type name.
    :param resp: The actual response object received.
    """

    def __init__(self, expected: str, resp: Any) -> None:
        """Initialize the error."""
        super().__init__(
            f"Unsupported response received: expected {expected},"
            f" received {type(resp)}"
        )


class UnsupportedTokenTypeError(SDKError):
    """Raised when the token exchange returned a non-Bearer token."""

    def __init__(self, token_type: str) -> None:
        """Initialize the error.

        :param token_type: The token type string received from the server.
        """
        super().__init__(
            f"Unsupported token received: expected Bearer, received {token_type}"
        )


class Receiver(ParentReceiver):
    """Receiver that performs a token exchange over gRPC.

    The receiver constructs an exchange request using the provided
    :class:`nebius.base.service_account.service_account.TokenRequester`
    and calls the :class:`TokenExchangeServiceClient.exchange` RPC.

    :param requester: Object that can construct the exchange request.
    :param service: Either a ready-made
        :class:`TokenExchangeServiceClient` or an awaitable resolving to
        one (useful when the channel is created asynchronously).
    :param max_retries: Maximum retry attempts for this receiver.
    """

    def __init__(
        self,
        requester: TokenRequester,
        service: TokenExchangeServiceClient | Awaitable[TokenExchangeServiceClient],
        max_retries: int = 2,
    ) -> None:
        """Initialize the receiver."""
        super().__init__()
        self._requester = requester
        self._svc = service
        self._max_retries = max_retries

        self._trial = 0

    def _raise_request_error(self, err: AioRpcError) -> None:
        """Convert a gRPC AioRpcError into a RequestError with diagnostics.

        The function extracts request and trace ids from the initial
        metadata and converts the gRPC status into the SDK's
        :class:`nebius.aio.service_error.RequestError` type.

        :param err: The original gRPC AioRpcError.
        :raises RequestError: always raised with enriched diagnostics.
        """
        initial_metadata = NebiusMetadata(err.initial_metadata())
        request_id = initial_metadata.get_one("x-request-id", "")
        trace_id = initial_metadata.get_one("x-trace-id", "")
        status = rpc_status.from_call(err)  # type: ignore
        from nebius.aio.service_error import RequestError, RequestStatusExtended

        if status is None:
            self._status = RequestStatusExtended(
                code=err.code(),
                message=err.details(),
                details=[],
                service_errors=[],
                request_id=request_id,
                trace_id=trace_id,
            )
            raise RequestError(self._status) from None

        self._status = RequestStatusExtended.from_rpc_status(  # type: ignore[unused-ignore]
            status,
            trace_id=trace_id,
            request_id=request_id,
        )
        raise RequestError(self._status) from None

    async def _fetch(
        self, timeout: float | None = None, options: dict[str, str] | None = None
    ) -> Token:
        """Perform the exchange RPC and return a :class:`Token`.

        :param timeout: Optional RPC timeout in seconds forwarded to the
            underlying gRPC stub.
        :param options: Optional map of request-specific options (currently
            unused).
        :returns: A :class:`Token` representing the exchanged access token.
        :raises nebius.aio.service_error.RequestError: when the RPC returns a service
            error.
        :raises UnsupportedResponseError: when the RPC returns an unexpected
            payload type.
        :raises UnsupportedTokenTypeError: when the received token type is not
            ``"Bearer"``.
        """
        self._trial += 1
        req = self._requester.get_exchange_token_request()

        now = datetime.now(timezone.utc)

        log.debug(f"fetching new token, attempt: {self._trial}, timeout: {timeout}")

        ret = None
        try:
            if isinstance(self._svc, Awaitable):
                self._svc = await self._svc
            ret = await self._svc.exchange(
                req,
                timeout=timeout,
                auth_options={OPTION_TYPE: Types.DISABLE},
            )
        except AioRpcError as e:
            self._raise_request_error(e)
        if not isinstance(ret, CreateTokenResponse):
            raise UnsupportedResponseError(CreateTokenResponse.__name__, ret)

        if ret.token_type != "Bearer":  # noqa: S105 — not a password
            raise UnsupportedTokenTypeError(ret.token_type)

        log.debug(
            f"token fetched: {sanitizer.sanitize(ret.access_token)},"
            f" expires in: {ret.expires_in} seconds."
        )
        return Token(
            token=ret.access_token, expiration=now + timedelta(seconds=ret.expires_in)
        )

    def can_retry(
        self,
        err: Exception,
        options: dict[str, str] | None = None,
    ) -> bool:
        """Decide whether the receiver should attempt another retry.

        The method honours the per-request :data:`OPTION_MAX_RETRIES` override
        if present and parses it as an integer. Invalid values are logged
        and ignored.
        """
        max_retries = self._max_retries
        if options is not None and OPTION_MAX_RETRIES in options:
            value = options[OPTION_MAX_RETRIES]
            try:
                max_retries = int(value)
            except ValueError as err:
                log.error(f"option {OPTION_MAX_RETRIES} is not valid integer: {err=}")
        if self._trial >= max_retries:
            log.debug("token max retries reached, cannot retry")
            return False
        return True


class Bearer(ParentBearer):
    """Bearer that creates receivers performing token exchange.

    The bearer accepts either a concrete gRPC channel or an awaitable
    resolving to one. The channel is wrapped in a
    :class:`TokenExchangeServiceClient` and used by receivers to perform
    the exchange RPC.

    :param requester: Object that knows how to build an exchange
        request for the current service account.
    :param channel: gRPC channel or awaitable resolving to one. If
        omitted, callers must call :meth:`set_channel` before
        obtaining a receiver.
    :param max_retries: Default retry attempts for receivers created
        by this bearer.

    Example
    -------

    Construct a bearer and use it to initialize the SDK::

        from asyncio import Future
        from nebius.sdk import SDK
        from nebius.aio.token.exchangeable import Bearer
        from nebius.base.service_account.pk_file import Reader as PKReader

        # Create requester from private key
        requester = PKReader(
            service_account_private_key_file_name="path/to/private_key.pem",
            service_account_id="your-service-account-id",
            service_account_public_key_id="your-public-key-id",
        )

        # Create a future for the channel that will be resolved with the SDK
        channel_future = Future()

        sdk = SDK(credentials=Bearer(requester=requester, channel=channel_future))

        # Resolve the future with the newly created SDK
        channel_future.set_result(sdk)
    """

    def __init__(
        self,
        requester: TokenRequester,
        channel: ClientChannelInterface | DeferredChannel | None = None,
        max_retries: int = 2,
    ) -> None:
        """Create an exchangeable bearer."""
        super().__init__()
        self._requester = requester
        self._max_retries = max_retries

        self._svc: (
            TokenExchangeServiceClient
            | Coroutine[Any, Any, TokenExchangeServiceClient]
            | None
        ) = None
        self.set_channel(channel)

    def set_channel(
        self,
        channel: ClientChannelInterface | DeferredChannel | None,
    ) -> None:
        """Set the gRPC channel or awaitable channel used for exchanges.

        The function accepts either a concrete :class:`ClientChannelInterface`
        or an awaitable resolving to one. When an awaitable is supplied,
        the bearer lazily constructs a stub coroutine that awaits the
        channel and then constructs the :class:`TokenExchangeServiceClient`.

        :param channel: The channel or awaitable resolving to a channel.
        """
        if isinstance(channel, Awaitable):  # type: ignore[unused-ignore]

            async def token_exchange_service_stub() -> TokenExchangeServiceClient:
                chan = await channel
                if not isinstance(chan, ClientChannelInterface):  # type: ignore[unused-ignore]
                    raise TypeError(
                        f"Expected ClientChannelInterface, got {type(chan)} instead."
                    )
                return TokenExchangeServiceClient(chan)

            self._svc = token_exchange_service_stub()

        elif channel is not None:
            self._svc = TokenExchangeServiceClient(channel)
        else:
            self._svc = None

    def receiver(self) -> Receiver:
        """Return a :class:`Receiver` that performs exchanges.

        :raises ValueError: if no channel has been configured on the
            bearer.
        :returns: A :class:`Receiver` instance.
        """
        if self._svc is None:
            raise ValueError("gRPC channel is not set for the bearer.")
        return Receiver(self._requester, self._svc, max_retries=self._max_retries)
